
<script type="text/plain" id="RenderMap.glsl">
//render image plane color(xyz) and id(w) to equirectangular map
attribute vec3 vpos;
uniform vec4 params[3];/*0.xyz = plane direction
1.xyz = plane tangent
2.xyz = plane bitangent
2.w = planeid/255*/
uniform sampler2D img;
varying vec2 uv;

#define PI 3.14159265358

void VERT() {
	uv = vpos.xy*vec2(PI,PI*.5);
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec3 rd = vec3(cos(uv.x),sin(uv.y),sin(uv.x));
	rd.xz *= sqrt(1.-rd.y*rd.y);
	
	float pd = dot(params[0].xyz,params[0].xyz), ad = dot(rd,normalize(params[0].xyz));
	if (ad > 0.) {
		pd /= ad;
		vec3 pp = rd*pd-params[0].xyz;
		vec2 lp = vec2(dot(pp,params[1].xyz),dot(pp,params[2].xyz));
		fragColor = vec4(texture(img,lp+.5).xyz,params[2].w);
		gl_FragDepth = min(pd/100.,1.);
		return;
	}
	discard;
}
</script>


<script type="text/plain" id="Distance.glsl">
//compute distance outputs vec4(1st weight, 1st nearest id, 2nd weight, 2nd nearest id)
attribute vec3 vpos;
uniform vec4 params;/*0.xy = offset
0.zw = interval*/
uniform sampler2D img;
varying vec2 uv;

#define PI 3.14159265358

void VERT() {
	uv = vpos.xy*.5+.5;
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	float baseId = texture(img,uv).w, dst1 = 1e8, id1 = 1., dst2 = 1e8, id2 = 1.;
	for (float ox = -params.x; ox <= params.x; ox += params.z) {
		for (float oy = -params.y; oy <= params.y; oy += params.w) {
			float tid = texture(img,uv+vec2(ox,oy)).w;
			if (tid != baseId) {
				float od = length(vec2(ox,oy)/params.xy);
				if (od < dst1) {
					if (id1 != tid) {
						dst2 = dst1;
						id2 = id1;
					}
					dst1 = od;
					id1 = tid;
					
				} else if (od < dst2 && id1 != tid) {
					dst2 = od;
					id2 = tid;
				}
			}
		}
	}
	dst1 = 1.-min(dst1,1.);
	dst2 = 1.-min(dst2,1.);
	
	float total = 1.+dst1+dst2;
	fragColor = vec4(dst1/total,id1,dst2/total,id2);
}
</script>


<script type="text/plain" id="Error.glsl">
//compute error between weighted planes and base plane
attribute vec3 vpos;
uniform vec4 params[3];/*0.xyz = plane direction
1.xyz = plane tangent
2.xyz = plane bitangent
2.w = planeid/255*/
uniform sampler2D img, map, weight;
varying vec4 uv;

#define PI 3.14159265358

void VERT() {
	uv = vec4(vpos.xy*vec2(PI,PI*.5), vpos.xy*.5+.5);
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec4 wt = texture(weight,uv.zw);
	float scale = 0.;
	if (abs(wt.y-params[2].w) < 1./512.) scale = wt.x;
	else if (abs(wt.w-params[2].w) < 1./512.) scale = wt.z;

	if (scale != 0.) {
		vec3 rd = vec3(cos(uv.x),sin(uv.y),sin(uv.x));
		rd.xz *= sqrt(1.-rd.y*rd.y);
	
		float pd = dot(params[0].xyz,params[0].xyz), ad = dot(rd,normalize(params[0].xyz));
		if (ad > 0.) {
			pd /= ad;
			vec3 pp = rd*pd-params[0].xyz;
			vec2 lp = vec2(dot(pp,params[1].xyz),dot(pp,params[2].xyz));
			fragColor = vec4(abs(texture(img,lp+.5).xyz-texture(map,uv.zw).xyz)*scale*2.,1.);
			return;
		}
	}
	fragColor = vec4(0);
}
</script>


<script type="text/plain" id="BaseImage.glsl">
//multiply main plane color by weight
attribute vec3 vpos;

uniform sampler2D img, weight;
varying vec2 uv;

void VERT() {
	uv = vpos.xy*.5+.5;
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec4 wt = texture(weight,uv);
	fragColor = vec4(texture(img,uv).xyz*(1.-(wt.x+wt.z)),1);
}
</script>


<script type="text/plain" id="AddPlane.glsl">
//add weighted plane
attribute vec3 vpos;
uniform vec4 params[3];/*0.xyz = plane direction
1.xyz = plane tangent
2.xyz = plane bitangent
2.w = planeid/255*/
uniform sampler2D img, weight;
varying vec4 uv;

#define PI 3.14159265358

void VERT() {
	uv = vec4(vpos.xy*vec2(PI,PI*.5), vpos.xy*.5+.5);
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec4 wt = texture(weight,uv.zw);
	float scale = 0.;
	if (abs(wt.y-params[2].w) < 1./512.) scale = wt.x;
	else if (abs(wt.w-params[2].w) < 1./512.) scale = wt.z;

	if (scale != 0.) {
		vec3 rd = vec3(cos(uv.x),sin(uv.y),sin(uv.x));
		rd.xz *= sqrt(1.-rd.y*rd.y);
	
		float pd = dot(params[0].xyz,params[0].xyz), ad = dot(rd,normalize(params[0].xyz));
		if (ad > 0.) {
			pd /= ad;
			vec3 pp = rd*pd-params[0].xyz;
			vec2 lp = vec2(dot(pp,params[1].xyz),dot(pp,params[2].xyz));
			fragColor = vec4(texture(img,lp+.5).xyz*scale,1);
			return;
		}
	}
	fragColor = vec4(0);
}
</script>


<script type="text/plain" id="RenderPreview.glsl">
//render preview of equirectangular map
attribute vec3 vpos;
uniform vec4 params[2];/*0.xy = camera yaw cos/sin
0.zw = camera pitch cos/sin

1.xy = aspect fov xy*/
uniform sampler2D map;
varying vec2 uv;

#define PI 3.14159265358

void VERT() {
	uv = vpos.xy*params[1].xy;
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec3 rd = normalize(vec3(uv,1));
	rd.yz *= mat2(params[0].zw,-params[0].w,params[0].z);
	rd.xz *= mat2(params[0].xy,-params[0].y,params[0].x);
	
	//sample equirectangular map
	fragColor = texture(map,vec2(atan(rd.z,rd.x)/(PI*2.)+.5,acos(-rd.y)/PI));
}
</script>


<script type="text/plain" id="CameraMap.glsl">
//render image plane color(xyz) and id(w) to camera screen with cylinder projection
attribute vec3 vpos;
uniform vec4 params[4];/*0.xyz = plane direction
0.w = camera yaw range
1.xyz = plane tangent
1.w = camera pitch range
2.xyz = plane bitangent
2.w = planeid/255,

3.xy = camera yaw cos/sin
3.zw = camera pitch cos/sin*/
uniform sampler2D img;
varying vec2 uv;

#define PI 3.14159265358

mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}

void VERT() {
	uv = vpos.xy*vec2(params[0].w,params[1].w);
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec3 rd = vec3(0,0,1);
	rd.yz *= r2d(uv.y);
	rd.xz *= r2d(uv.x);
	rd.yz *= mat2(params[3].z,params[3].w,-params[3].w,params[3].z);
	rd.xz *= mat2(params[3].x,params[3].y,-params[3].y,params[3].x);

	float pd = dot(params[0].xyz,params[0].xyz), ad = dot(rd,normalize(params[0].xyz));
	if (ad > 0.) {
		pd /= ad;
		vec3 pp = rd*pd-params[0].xyz;
		vec2 lp = vec2(dot(pp,params[1].xyz),dot(pp,params[2].xyz));
		fragColor = vec4(texture(img,lp+.5).xyz,params[2].w);
		gl_FragDepth = min(pd/100.,1.);
		return;
	}
	discard;
}
</script>

<script type="text/plain" id="AddPlaneCube.glsl">
//add weighted plane to cylinder view
attribute vec3 vpos;
uniform vec4 params[4];/*0.xyz = plane direction
0.w = camera yaw range
1.xyz = plane tangent
1.w = camera pitch range
2.xyz = plane bitangent
2.w = planeid/255

3.xy = camera yaw cos/sin
3.zw = camera pitch cos/sin*/
uniform sampler2D img, weight;
varying vec4 uv;

#define PI 3.14159265358

mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}

void VERT() {
	uv = vec4(vpos.xy*vec2(params[0].w,params[1].w), vpos.xy*.5+.5);
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec4 wt = texture(weight,uv.zw);
	float scale = 0.;
	if (abs(wt.y-params[2].w) < 1./512.) scale = wt.x;
	else if (abs(wt.w-params[2].w) < 1./512.) scale = wt.z;

	if (scale != 0.) {
		vec3 rd = vec3(0,0,1);
		rd.yz *= r2d(uv.y);
		rd.xz *= r2d(uv.x);
		rd.yz *= mat2(params[3].z,params[3].w,-params[3].w,params[3].z);
		rd.xz *= mat2(params[3].x,params[3].y,-params[3].y,params[3].x);
	
		float pd = dot(params[0].xyz,params[0].xyz), ad = dot(rd,normalize(params[0].xyz));
		if (ad > 0.) {
			pd /= ad;
			vec3 pp = rd*pd-params[0].xyz;
			vec2 lp = vec2(dot(pp,params[1].xyz),dot(pp,params[2].xyz));
			fragColor = vec4(texture(img,lp+.5).xyz*scale,1);
			return;
		}
	}
	fragColor = vec4(0);
}
</script>

<script type="text/plain" id="CubeMap.glsl">
//add camera result to cubemap
attribute vec3 vpos;
uniform vec4 params[2];/*0.xy = cam yaw cos/sin
0.zw = cam pitch cos/sin

1.x = camera yaw range
1.y = camera pitch range
1.w = cube face id*/
uniform sampler2D img;
varying vec2 uv;
varying vec3 urd;

#define PI 3.14159265358


void VERT() {
	uv = vpos.xy;
	float fid = params[1].w;
	if (fid == 0.) urd = vec3(-1,vpos.y,vpos.x);//face 0 -X
	else if (fid == 1.) urd = vec3(1,vpos.y,-vpos.x);//face 1 +X
	else if (fid == 2.) urd = vec3(vpos.x,-1,vpos.y);//face 2 -Y
	else if (fid == 3.) urd = vec3(vpos.x,1,-vpos.y);//face 3 +Y
	else if (fid == 4.) urd = vec3(-vpos.x,vpos.y,-1);//face 4 -Z
	else urd = vec3(vpos.xy,1);//face 5 +Z
	gl_Position = vec4(vpos,1);
}
void FRAG() {
	vec3 rd = normalize(urd);
	//reproject cylinder camera
	rd.xz *= mat2(params[0].x,-params[0].y,params[0].y,params[0].x);
	rd.yz *= mat2(params[0].z,-params[0].w,params[0].w,params[0].z);
	vec2 ang = vec2(atan(rd.x,rd.z),acos(-rd.y)-PI*0.5)/params[1].xy;
	
	if (max(abs(ang.x),abs(ang.y)) < 1.) fragColor = texture(img,ang*.5+.5);
	else fragColor = vec4(0,0,0,1);
}
</script>
